package net.sf.corn.converter.json;

import java.lang.reflect.Array;

import net.sf.corn.converter.ConversionException;
import net.sf.corn.converter.ObjectReferenceMap;
/**
 * Converter for Arrays 
 * @author Serhat Dirik
 *
 */
class CnvArray extends AbstractJsonConverter {

	CnvArray() {

	}

	@Override
	protected JsTypeObject inToTarget(Object o, ObjectReferenceMap referenceMap)
			throws ConversionException {
		JsTypeList jsonList = new JsTypeList(o.getClass());
		JsonConverter transformer = new JsonConverter();
		for (int i = 0; i < Array.getLength(o); i++) {
			Object object = Array.get(o, i);
			jsonList.add(transformer.toTargetType(object, referenceMap));
		}
		return jsonList;
	}

	@Override
	protected Object inToJava(Class<?> clazz, JsTypeObject obj)
			throws ConversionException {
		if (!obj.isList()) {
			throw new ConversionException("Only list type is allowed");
		}
		JsTypeList jso = (JsTypeList) obj;
		JsonConverter transformer = new JsonConverter();
		Object arr = null;
		try {
			Class<?> componentType = jso.getJavaClass();
			if (componentType.isArray()) {
				String class_name = componentType.getCanonicalName();
				class_name = class_name.substring(0, class_name.indexOf('['));
				componentType = Class.forName(class_name);
			}
			arr = Array.newInstance(componentType, jso.size());
			for (int i = 0; i < jso.size(); i++) {
				Array.set(arr, i, transformer.toJava(jso.get(i)));
			}
		} catch (Exception e) {
			throw new ConversionException(e);
		}

		return arr;
	}

	@Override
	public boolean isConvertible(Class<?> clazz) {
		if (!clazz.isArray())
			return false;
		JsonConverter transformer = new JsonConverter();
		return clazz.getComponentType().equals(Object.class) ? true
				: transformer.isConvertible(clazz.getComponentType());
	}

	@Override
	public boolean isSubjectForCircularReferenceCheck() {
		return true;
	}

}
