package net.sf.corn.converter.json;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import net.sf.corn.converter.ConversionException;
import net.sf.corn.converter.ObjectReferenceMap;

class CnvBean extends AbstractJsonConverter {

	CnvBean() {
	}

	@Override
	protected JsTypeObject inToTarget(Object o, ObjectReferenceMap referenceMap)
			throws ConversionException {
		JsTypeComplex jsonObject = new JsTypeComplex(o.getClass());
		JsonConverter transformer = new JsonConverter();
		if (referenceMap == null)
			referenceMap = new ObjectReferenceMap();
		List<String> capturedFields = new ArrayList<String>(5);
		// Try to access test.bean data
		BeanInfo bi = null;
		try {
			bi = Introspector.getBeanInfo(o.getClass(),
					Introspector.USE_ALL_BEANINFO);
		} catch (IntrospectionException e1) {
			bi = null;
		}
		PropertyDescriptor[] descriptors = bi.getPropertyDescriptors();
		for (PropertyDescriptor pDesc : descriptors) {
			Object propVal = null;
			String propName = pDesc.getName();
			if (propName.equals("class") || propName.equals("bytes"))
				continue;
			if (capturedFields.contains(propName))
				continue;
			if (pDesc.getReadMethod() == null)
				continue;
			try {
				propVal = pDesc.getReadMethod().invoke(o, new Object[] {});
			} catch (Exception e) {
				continue;
			}
			JsTypeObject beanProp;
			beanProp = transformer.toTargetType(propVal, referenceMap);
			jsonObject.put(propName, beanProp);
			capturedFields.add(propName);
		}
		// Try to access object fields which are not accessed through
		// test.bean methods
		for (Field field : o.getClass().getDeclaredFields()) {
			String fieldName = field.getName();
			if (fieldName.equals("class") || fieldName.equals("bytes"))
				continue;
			if (Modifier.isTransient(field.getModifiers()))
				continue;
			if (capturedFields.contains(fieldName))
				continue;
			// if(! Modifier.isPublic(field.getModifiers()) ) continue;
			try {
				if (!field.isAccessible()) {
					field.setAccessible(true);
				}
			} catch (Exception e) {
				continue;
			}
			Object fieldVal;
			try {
				fieldVal = field.get(o);
			} catch (Exception e) {
				continue;
			}
			JsTypeObject beanProp;
			beanProp = transformer.toTargetType(fieldVal, referenceMap);
			jsonObject.put(fieldName, beanProp);
			capturedFields.add(fieldName);

		}

		return jsonObject;
	}

	@Override
	protected Object inToJava(Class<?> clazz, JsTypeObject obj)
			throws ConversionException {

		if (!obj.isComplex()) {
			throw new ConversionException("Only list type is allowed");
		}
		JsTypeComplex jso = (JsTypeComplex) obj;

		List<String> capturedFields = new ArrayList<String>(5);
		Object bean;
		try {
			bean = clazz.newInstance();
		} catch (InstantiationException e) {

			throw new ConversionException(clazz.getName()
					+ " doesn't have a default public contructor!", e);
		} catch (IllegalAccessException e) {
			throw new ConversionException(e);
		}
		PropertyDescriptor[] propertyDescriptors = getBeanProperties(clazz);

		JsonConverter transformer = new JsonConverter();
		for (String key : jso.keys()) {
			JsTypeObject beanProp;
			beanProp = jso.get(key);
			if (beanProp == null) {
				throw new ConversionException("bean property data (" + key
						+ ") is missing");
			}

			boolean valueAssigned = false;
			boolean breakOnError = true;
			if (propertyDescriptors != null) {
				for (PropertyDescriptor pDesc : propertyDescriptors) {
					String propName = pDesc.getName();
					if (propName.equals("class") || propName.equals("bytes"))
						continue;
					if (!propName.equals(key))
						continue;
					if (capturedFields.contains(propName))
						continue;
					Object beanPropVal = null;
					boolean notConverted = true;

					try {
						// Transform by using target class
						beanPropVal = transformer.toJava(
								pDesc.getPropertyType(), beanProp);
						notConverted = false;
					} catch (Exception e) {
						// do nothing yet
					}
					if (notConverted) {
						try {
							// Transform
							Object tempVal = transformer.toJava(beanProp);
							if (tempVal == null) {
								beanPropVal = tempVal;
								notConverted = false;
							} else if (pDesc.getPropertyType()
									.isAssignableFrom(tempVal.getClass())) {
								beanPropVal = pDesc.getPropertyType().cast(
										tempVal);
								notConverted = false;
							} else {
								continue;
							}
						} catch (Exception e1) {
							continue;// Skip trying to set
						}
					}

					try {
						if (pDesc.getWriteMethod() == null)
							break;
						pDesc.getWriteMethod().invoke(bean,
								new Object[] { beanPropVal });
						capturedFields.add(key);
						valueAssigned = true;
					} catch (Exception e) {

					}
					break;
				}
			}

			if (!valueAssigned) {
				// Try to assign field values which are not accessed through
				// test.bean methods
				Field[] fields = getBeanFields(clazz);
				for (Field field : fields) {
					try {
						String fieldName = field.getName();
						if (fieldName.equals("class")
								|| fieldName.equals("bytes"))
							continue;
						if (Modifier.isFinal(field.getModifiers())) {
							valueAssigned = true;
							continue;
						}
						if (!fieldName.equals(key))
							continue;
						if (capturedFields.contains(fieldName))
							continue;
						if (Modifier.isTransient(field.getModifiers())) {
							valueAssigned = true;
							continue;
						}

						Object beanPropVal = null;
						boolean notConverted = true;
						beanPropVal = transformer.toJava(field.getType(),
								beanProp);
						notConverted = false;
						if (notConverted) {
							try {
								// Transform
								Object tempVal = transformer.toJava(beanProp);
								if (tempVal == null) {
									beanPropVal = tempVal;
									notConverted = false;
								} else if (field.getType().isAssignableFrom(
										tempVal.getClass())) {
									beanPropVal = field.getType().cast(tempVal);
									notConverted = false;
								} else {
									break; // Value is not assigned
								}
							} catch (Exception e1) {
								break; // Value is not assigned
							}
						}

						// if(! Modifier.isPublic(field.getModifiers()) )
						// continue;
						try {
							if (!field.isAccessible()) {
								breakOnError = false;
								field.setAccessible(true);
							}
							field.set(bean, beanPropVal);
							valueAssigned = true;
						} catch (Exception e) {
							break;
						}

						capturedFields.add(fieldName);
						break;
					} catch (Exception e) {
					}

				}
			}

			if (!valueAssigned && breakOnError) {
				throw new ConversionException(
						"No valid setter method or accesible bean property defined for "
								+ key + " property in " + clazz.getName()
								+ " class");
			}
		}

		return bean;

	}

	private final static Class<?>[] EMPTY_FORMAL_PARAM = new Class[] {};

	@Override
	public boolean isConvertible(Class<?> clazz) {
		boolean retVal = false;
		// Beans must have public default constructor
		try {
			Constructor<?> c = clazz.getConstructor(EMPTY_FORMAL_PARAM);
			retVal = (c != null);
		} catch (Throwable e) {
			// e.printStackTrace();
		}
 
		return retVal;
	}

	@Override
	public boolean isSubjectForCircularReferenceCheck() {
		return true;
	}

	/**
	 * Gives bean properties as array
	 * 
	 * @param bean
	 * @return PropertyDescriptor[]
	 */
	private static PropertyDescriptor[] getBeanProperties(Class<?> clazz) {
		if (clazz == null)
			return null;
		BeanInfo bi;
		try {
			bi = Introspector.getBeanInfo(clazz, Introspector.USE_ALL_BEANINFO);
		} catch (IntrospectionException e) {
			return null;
		}
		PropertyDescriptor[] descriptors = bi.getPropertyDescriptors();
		Class<?> superClazz = clazz.getSuperclass();
		PropertyDescriptor[] additional = null;
		if (superClazz != null) {
			additional = getBeanProperties(superClazz);
		}
		if (additional != null) {
			Set<PropertyDescriptor> dSet = new HashSet<PropertyDescriptor>();
			dSet.addAll(Arrays.asList(descriptors));
			dSet.addAll(Arrays.asList(additional));
			descriptors = new PropertyDescriptor[dSet.size()];
			int i = 0;
			for (PropertyDescriptor propertyDescriptor : dSet) {
				descriptors[i] = propertyDescriptor;
				i++;
			}
		}

		return descriptors;
	}

	private static Field[] getBeanFields(Class<?> clazz) {
		if (clazz == null)
			return null;
		Field[] fields = clazz.getDeclaredFields();
		Class<?> superClazz = clazz.getSuperclass();
		Field[] additional = null;
		if (superClazz != null) {
			additional = getBeanFields(superClazz);
		}
		if (additional != null) {
			Set<Field> dSet = new HashSet<Field>();
			dSet.addAll(Arrays.asList(fields));
			dSet.addAll(Arrays.asList(additional));
			fields = new Field[dSet.size()];
			int i = 0;
			for (Field field : dSet) {
				fields[i] = field;
				i++;
			}
		}
		return fields;

	}

}
