package net.sf.corn.converter.json;

import java.util.TimeZone;

import net.sf.corn.converter.ConversionException;
import net.sf.corn.converter.ObjectReferenceMap;
import static net.sf.corn.converter.json.IJsonConstants.*;

class CnvCalendar extends AbstractJsonConverter {

	CnvCalendar() {
		getConvertibles().add(java.util.Calendar.class);
		getConvertibles().add(java.util.GregorianCalendar.class);
	}

	@Override
	protected JsTypeObject inToTarget(Object o, ObjectReferenceMap referenceMap)
			throws ConversionException {
		JsTypeComplex jsonObject = new JsTypeComplex(o.getClass());
		java.util.Calendar cal = (java.util.Calendar) o;
		JsonConverter transformer = new JsonConverter();
		jsonObject.put(ATTR_TIME,
				transformer.toTargetType(cal.getTimeInMillis(), referenceMap));
		jsonObject.put(ATTR_TIMEZONEID, transformer.toTargetType(cal
				.getTimeZone().getID(), referenceMap));

		return jsonObject;
	}

	@Override
	protected Object inToJava(Class<?> clazz, JsTypeObject object)
			throws ConversionException {
		if (!object.isComplex()) {
			throw new ConversionException("Only complex type is allowed");
		}
		JsTypeComplex jso = (JsTypeComplex) object;
		if ( !jso.has(ATTR_TIME)){
			throw new ConversionException(ATTR_TIME + " attribute not found");
		}
		Long time = 0l;
		String timeZoneId = "";
		time = Long.valueOf(jso.get(ATTR_TIME).toStringValue());
		if ( jso.has(ATTR_TIMEZONEID)){
			timeZoneId = jso.get(ATTR_TIMEZONEID).toStringValue();
		}else{
			timeZoneId = TimeZone.getDefault().getID();
		}
		
		java.util.TimeZone zone = java.util.TimeZone.getTimeZone(timeZoneId);
		if (clazz.equals(java.util.Calendar.class)) {
			java.util.Calendar cal = java.util.Calendar.getInstance();
			cal.setTimeZone(zone);
			cal.setTimeInMillis(time);
			return cal;
		} else if (clazz.equals(java.util.GregorianCalendar.class)) {
			java.util.GregorianCalendar cal = new java.util.GregorianCalendar(
					zone);
			cal.setTimeInMillis(time);
			return cal;
		} else {
			try {
				java.util.Calendar cal = (java.util.Calendar) clazz
						.newInstance();
				cal.setTimeZone(zone);
				cal.setTimeInMillis(time);
				return cal;
			} catch (InstantiationException e) {
				throw new ConversionException(e);
			} catch (IllegalAccessException e) {
				throw new ConversionException(e);
			}

		}
	}

	@Override
	public boolean isConvertible(Class<?> clazz) {
		boolean ret = checkIfConvertibleFromRegisteredTypes(clazz);
		if (!ret) {
			ret = java.util.Calendar.class.isAssignableFrom(clazz);
		}
		return ret;
	}

	@Override
	public boolean isSubjectForCircularReferenceCheck() {
		return false;
	}
}
